/world
{
	turf = /turf;
	mob = /mob/player;

	name = "Treasure Seeker";
	hub = 969;

	version = 1;

	New()
	{
		LoadBanList();
		DoGenMap();
		ClassDescs();
		InitPotionList();

		spawn(320) players << "<B>Map generation complete!</B>";
	}
}

/var
{
	dirlist[] = list(NORTH, EAST, SOUTH, WEST);
	players[] = list();
	allplayers[] = list();
	combats[] = list();
}

/area
{
	icon = 'blank.dmi';
	layer = TURF_LAYER;
}

/turf
{
	layer = AREA_LAYER;

	var
	{
		combat/combat;
		trap/trap;
		special/special;

		stairs = null;
		exits;
	}

	icon = 'room.dmi';

	Enter(atom/movable/O)
	{
		if (O.density == 0)
			return 1;

		if (O.z != src.z)
			return 1;

		if (!(src.exits & get_dir(src, O.loc)))
			return 0;
		else if (!(O.loc:exits & get_dir(O.loc, src)))
			return 0;

		return 1;
	}

	Exit(atom/movable/O)
	{
		if (ismob(O) && O:opponent != null)
		{
			if (O in players) O << "You're currently engaged in combat. Use the flee verb if you really want to leave.";
			return 0;
		}

		return 1;
	}

	proc
	{
		UpdateAppearance()
		{
			src.icon_state = num2text(src.exits);

			src.overlays.Cut();

			if (src.stairs == "up")
				src.overlays += 'stairup.dmi';
			else if (src.stairs == "down")
				src.overlays += 'stairdown.dmi';
			else if (src.stairs == "both")
				src.overlays = src.overlays + 'stairup.dmi' + 'stairdown.dmi';

			var/numplayers = 0;

			if (src.combat != null)
			{
				src.overlays += 'combat.dmi';
			}
			else
			{
				for (var/atom/A in src.contents)
				{
					if (istype(A, /mob/monster) || istype(A, /mob/companion))
					{
						if (!('people.dmi' in src.overlays))
							src.overlays += 'people.dmi';
					}
					else if (istype(A, /mob/player))
					{
						numplayers++;
						if (numplayers > 1 && !('people.dmi' in src.overlays)) src.overlays += 'people.dmi';
					}
				}
			}
			for (var/atom/A in src.contents)
			{
				if (isobj(A))
				{
					if (!('treasure.dmi' in src.overlays)) src.overlays += 'treasure.dmi';
				}
			}
		}

		Reveal(mob/player/P)
		{
			src.UpdateAppearance();

			if (!P) return;

			if (P.cmap["[src.x],[src.y],[src.z]"])
				del(P.cmap["[src.x],[src.y],[src.z]"]);

			P.cmap["[src.x],[src.y],[src.z]"] = image(src, src, layer=AREA_LAYER + 1);
			P << P.cmap["[src.x],[src.y],[src.z]"];

			if (P.image)
			{
				del (P.image);
				P.image = image(P, P, layer=MOB_LAYER);
				P << P.image;
			}

			for (var/mob/M in src)
			{
				if (M != P)
				{
					if (M.opponent && M.opponent != P)
						P << "You see \a [M] fightin \a [M.opponent].";
					else if (M.opponent == P)
						P << "You see \a [M] fighting you!";
					else
						P << "You see \a [M].";
				}
			}

			for (var/obj/O in src)
			{
				P << "You see \a [O] on the ground.";
			}
		}

		RevealAll()
		{
			src.UpdateAppearance();

			for (var/mob/player/P in src)
			{
				if (!istype(P)) continue;

				if (P.cmap["[src.x],[src.y],[src.z]"])
					del(P.cmap["[src.x],[src.y],[src.z]"]);

				P.cmap["[src.x],[src.y],[src.z]"] = image(src, src, layer=AREA_LAYER + 1);
				P << P.cmap["[src.x],[src.y],[src.z]"];

				if (P.image)
				{
					del (P.image);
					P.image = image(P, P, layer=MOB_LAYER);
					P << P.image;
				}
			}
		}
	}
}

/mob
{
	var
	{
		attack;
		lore;
		charisma;
		curlife;
		life;
		flee;

		turf/move;

		mob/opponent = null;

		undead=0;
		lucky = 0;
		accuracy = 0;
		blessed = 0;

		bounty = 0;

		image/image;
		image/frame;
		frames[] = list();

		score = 0;
		xp = 0;
		max_xp = 4;

		defend = 0;
		defense = "is unharmed by";
		nat_defense = "is unharmed by";

		block = 0;
		blocks = "deflects";
		nat_blocks = "deflects";

		chat_time;
		class;

		adds;
		lore_adds;
		afk = 0;
		def_adds;

		companions[] = list();

		combat/combat;

		color;

		obj/item/weapon;
		obj/item/sword;
		obj/item/shield;
		obj/item/armor;
		misc_equipped[] = list();
	}

	verb
	{
		examine()
		{
			set src in usr.loc;

			usr << "<B>[src.name]:  [src:curlife]/[src:life] life</B>";
			if (istype(src, /mob/player))
			{
				if (src:companions.len > 0)
				{
					usr << "[src] is accompanied by \...";

					for (var/i=1, i<=src:companions.len, i++)
					{
						if (i == 1)
							usr << "\a [src:companions[i]]\...";
						else if (i == src:companions.len && src:companions.len == 2)
							usr << " and \a [src:companions[i]].";
						else if (i == src:companions.len)
							usr << ", and \a [src:companions[i]].";
						else
							usr << ", \a [src:companions[i]]\...";
					}
				}
			}

			usr << src.desc;
		}

		attack()
		{
			set src in usr.loc;

			if (istype(src, /mob/companion))
			{
				usr << "Sorry, attacking neutral NPCs isn't allowed. Try greeting them instead.";
				return;
			}

			usr:Attack(src);
		}
	}

	proc/Die()
	{
		if (src.opponent)
		{
			if (src.opponent.class == "Zombie" && src.undead != 1)
			{
				src.opponent << "<font color=#904040>You feast upon the still-warm flesh of [src]!</font>";
				if (!istype(src,/mob/player))
					src.opponent.curlife = min(src.opponent.curlife + src.life, src.opponent.life);
				else
					src.opponent.curlife = min(src.opponent.curlife + 5, src.opponent.life);
			}
			else if (src.undead != 1)
			{
				for (var/obj/item/I in src.opponent.contents)
				{
					if (I && I.type == /obj/item/rare/ring_of_vampirism && I.equipped == 1)
					{
						if (I.Magic_Use(src.opponent))
						{
							src.opponent << "<font color=#C06060>You feel [src]'s life force flowing into you through the Ring of Vampirism...</font>";
							src.opponent.curlife = min(src.opponent.curlife + 1, src.opponent.life);
							break;
						}
					}
				}
			}

			if (istype(src.opponent, /mob/player) && istype(src, /mob/monster))
			{
				src.opponent.xp++;
				if (src.opponent.xp >= src.opponent.max_xp)
				{
					src.opponent.xp = 0;
					src.opponent.max_xp++;
					if (src.opponent.curlife < src.opponent.life)
						src.opponent.curlife = min(src.opponent.curlife + 2, src.opponent.life);
					else
					{
						src.opponent.life++;
						src.opponent.curlife = src.opponent.life;
					}
					src.opponent << "<font color=#C0C080>Exhilirated by your successes, you continue on your quest with a renewed sense of vigor!</font>";
				}
			}

			if (istype(src, /mob/monster) && src.opponent.class == "Bounty Hunter")
			{
				src.opponent.score += 10;
			}

			if (src.bounty && istype(src.opponent, /mob/player))
			{
				src.opponent.score += src.bounty;
			}

			if (src.type == /mob/monster/guardian && istype(src.opponent, /mob/player))
			{
				for (var/obj/item/I in src.contents)
				{
					I.loc = src.opponent;
					src.opponent.score += I.worth;
				}

				players << "<B><font color=#A0A000>[src.opponent] has found the Legendary Treasure Hoard!</font></B><font color=#B0B000> The game will end when someone successfully takes the Hoard out of the dungeon, so try to hurry back up to the surface to cash in what you have before it's too late--or track down [src.opponent] and claim the Hoard for yourself!</font>";
			}
		}

		if (src.bounty)
			src.bounty = 0;

		if (!(src in players))
		{
			if (istype(src.loc, /turf) && src.contents.len)
			{
				for (var/obj/item/I in src.contents)
				{
					I.loc = src.loc;
				}
			}
			del(src);
		}
		else
		{
			players << "<b>[src] is killed!</b>";
			for (var/obj/item/I in src.contents)
			{
				if (I.worth == 0)
					del (I);
				else
				{
					if (src.class == "Merchant")
						I.worth = round(I.worth * 5 / 6, 1);
					I.loc = src.loc;
				}
			}

			for (var/I in src:cmap)
			{
				del (src:cmap[I]);
			}

			var/newscore = src.score - src:pturn * 5;
			if (newscore > 0)
			{
				var/score/S = new();
				S.owner = src.name;
				S.total = newscore;
				scores.Add(S);
			}

			src.contents.Cut();
			src.loc = null;
			spawn(10) src:SelectClass();
		}
	}

	density = 1;
	layer = AREA_LAYER;


	verb
	{
		say(chat as text)
		{

			if (length(chat) > 250)
			{
				spawn(1) usr:say (copytext(chat, 250, 0));
				chat = copytext(chat, 1, 250);
			}
			if (usr.chat_time > world.time)
			{
				if (usr.chat_time > world.time + 30)
				{
					usr << "Booted for spamming.";
					(players - usr) << "[usr] was booted for spam.";
					del (usr);
				}
				spawn(usr.chat_time - world.time) players << html_encode("[usr]: [chat]");
				if (usr.chat_time > world.time + 10)
					usr.chat_time += 8;
				else
					usr.chat_time += 6;
			}
			else
			{
				if (src != tcarrier)
					players << "<font color=[fontlist[usr.color]]><b>[usr]</b></font>\..."
				else
					players << "<font color=[fontlist[13]]><b>[usr]</b></font>\..."
				players << html_encode(": [chat]");
				usr.chat_time = world.time + 6;
			}
		}

		who()
		{
			usr << "Players online:";
			for (var/mob/player/P in players)
			{
				if (P == tcarrier)
					usr << "<font color=[fontlist[13]]><b>[P.name]</b></font>";
				else
					usr << "<font color=[fontlist[P.color]]><b>[P.name]</b></font>";
			}
		}

		high_scores()
		{
			if (scores.len == 0)
				usr << "<B>No scores yet recorded.</B>";
			else
				ShowScores(usr);
		}

		afk()
		{
			if (src.afk == 1)
			{
				src.afk = -2;
				src.name = src.key;
				if (src.class)
					src.name += " the [src.class]";

				players << "[src] is back from AFK.";
				spawn(50) src.afk = 0;
			}
			else if (src.afk == 0)
			{
				src.afk = -2;
				players << "[src] is AFK.";
				src.name += " (AFK)";
				spawn(50) src.afk = 1;
			}
		}
	}

	player
	{

		Login()
		{
			src.name = src.key;

			if (src.client && src.client.address == null)
			{
				src.verbs += typesof(/mob/host/verb);
				host = src;
			}

			if (src in players)
				players.Remove(src);

			if (!(src in players))
				players.Add(src);

			if (src.type == /mob/player)
			{
				src.loc = null;
				sleep(10);
				src.SelectClass();
			}
			else
			{
				src.image = image(src, src);
				src << src.image;
				src << src.frame;
			}

			..();
		}

		Logout()
		{
			if (src in players)
			{
				for (var/obj/item/I in src.contents)
				{
					if (I.worth == 0)
						del (I);
					else
						I.loc = src.loc;
				}

				spawn(1) del (src);
			}

			players.Remove(src);

			..();
		}

		var
		{
			cmap[] = list();
			carrier;
			pturn = 0;
		}

		verb
		{
			look()
			{
				if (usr.loc)
					usr.loc:Reveal(usr);
			}

			flee()
			{
				if (!usr.combat)
					usr << "You're not engaged in any combat!";
				else if (!usr.flee)
					Flee();
			}


			ascend()
			{
				if (usr.loc:stairs != "up" && usr.loc:stairs != "both")
				{
					usr << "There aren't any stairs up here!";
					return;
				}
				else if (world.time < 320)
				{
					src << "The map is still being generated, please wait.";
					return 0;
				}
				else if (usr.opponent)
				{
					src << "You're currently engaged in combat. Use the flee verb if you really want to escape.";
					return 0;
				}

				if (usr.z == 1)
				{
					if (alert("Are you sure you want to leave the dungeon and cash in all your treasure?", "Confirm Exit", "Yes", "No") == "No")
						return;
					else
					{
						players << "[usr] exits the dungeon with a total score of [usr.score]!";

						for (var/obj/C in src.contents)
						{
							if (C.type == /obj/item/treasure_hoard)
							{
								DoEndGame();
								return;
							}

							C.Move(null);
							usr.contents.Remove(C);

						}

						var/mob/player/N = new();
						N.key = usr.key;
						N.name = usr.name;

						var/score/S = new();
						S.owner = src.name;
						S.total = src.score;
						scores.Add(S);

						usr.contents.Cut();

						for (var/I in src.cmap)
						{
							del (src.cmap[I]);
						}

						del(src);
					}
				}
				else
				{
					usr.move = locate(usr.x, usr.y, usr.z - 1);
				}
			}

			suicide()
			{
				if (alert("Are you sure?", "Paging Dr. Kevorkian", "Yes", "No") == "Yes")
				{
					usr.Die();

					if (usr.combat)
					{
						if (usr.opponent)
							usr.opponent.opponent = null;

						combats.Remove(usr.combat);
						del(usr.combat);
					}
				}
			}

			descend()
			{
				if (usr.loc:stairs != "down" && usr.loc:stairs != "both")
				{
					usr << "There aren't any stairs going down here!";
					return;
				}
				else if (world.time < 320)
				{
					src << "The map is still being generated, please wait.";
					return 0;
				}
				else if (usr.opponent)
				{
					src << "You're currently engaged in combat. Use the flee verb if you really want to escape.";
					return 0;
				}

				usr.move = locate(usr.x, usr.y, usr.z + 1);
			}
		}

		proc
		{
			Attack(mob/M)
			{
				if (src == M && src == usr)
				{
					usr << "Nooooooo! You've got too much to live for!";
					return;
				}

				if (M.opponent)
				{
					if (M.opponent == usr)
					{
						usr << "You're already fighting [M]!";
						return;
					}

					if (istype(M, /mob/monster))
					{
						usr << "Unfortunately, the local Adventurer's Union strictly prohibits attacking a monster currently being fought by a fellow union member.";
						return;
					}
					else
					{
						usr << "The Adventurer's Union has passed an ordinance stating that no union member may attack a fellow adventurer who's already engaged in combat.";
						return;
					}
				}
				else if (src.opponent)
				{
					src << "You're already fighting!";
					return;
				}
				else
				{
					if (usr.undead == 1 && M.class == "Priest")
					{
						usr << "Some holy force prevents you from attacking [M]!";
						return;
					}

					usr.loc.contents - M - usr << "[usr] lunges at [M]!";
					M << "[usr] grins evilly and attacks you!";
					usr << "You attack [M]!";
					usr.opponent = M;
					M.opponent = usr;

					var/combat/newcombat = new/combat();
					newcombat.turf = usr.loc;
					newcombat.sides.Add(usr);
					newcombat.sides.Add(M);
					usr.loc:combat = newcombat;
					M.combat = newcombat;
					usr.combat = newcombat;
					newcombat.turf.RevealAll();
				}
			}

			Flee()
			{
				if (usr.loc == null)
					return;

				if (usr.loc:exits == 0)
				{
					usr << "Oh no, you're trapped!";
					return;
				}
				usr.move = null;
				while (!usr.move)
				{
					usr.move = get_step(usr.loc, 2 ** rand(0, 3));
					if (!usr.move || !usr.move.Enter(usr))
						usr.move = null;
				}

				if (!usr.flee)
					usr.flee = 1;
			}
		}
	}
}

/client
{
	Northeast()
	Northwest()
	Southeast()
	Southwest()

	Move(turf/loc, dir)
	{
		if (src.mob.type == /mob/player)
			return 0
		else if (world.time < 320)
		{
			src << "The map is still being generated, please wait.";
			return 0;
		}
		else
		{
			if (src.mob.class == "Monk" && src.mob:med_turn >= turn)
			{

				usr << "You can't move while meditating!";
				return 0;
			}

			src.mob.flee = 0;

			if (loc && loc.Enter(src.mob) && src.mob.loc.Exit(src.mob))
				src.mob.move = loc;
			else
				src.mob.move = null;
		}
	}

	Center()
	{
		src.mob.move = null;
	}

	Del()
	{
		if (src.mob)
			world << "<B>[src] leaves.</B>";

		..();
	}

	New()
	{
		if (src.address in ban_list)
		{
			if (!src.key in ban_list["[src.address]"])
			{
				var/blist[] = ban_list["[src.address]"];
				blist.Add(src.key);
			}

			host << "Banned connection attempt by [src.key] from [src.address].";
			del (src);
		}
		else
		{
			for (var/i=1, i<=ban_list.len, i++)
			{
				if (src.key in ban_list[i])
				{
					host << "Banned connection attempt by [src.key] from [src.address].";
					del(src);
				}
			}
		}

		src << browse('temphelp.htm');

		spawn(1) world << "<B>[src] has entered the world.</B>";

		..();
	}
}